package com.oppwa.mobile.connect.demo.activity

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.PersistableBundle
import androidx.appcompat.app.AlertDialog
import com.oppwa.mobile.connect.demo.R
import com.oppwa.mobile.connect.demo.common.Constants
import com.oppwa.mobile.connect.demo.databinding.ActivityCustomUiBinding
import com.oppwa.mobile.connect.exception.PaymentError
import com.oppwa.mobile.connect.exception.PaymentException
import com.oppwa.mobile.connect.payment.CheckoutInfo
import com.oppwa.mobile.connect.payment.card.CardPaymentParams
import com.oppwa.mobile.connect.provider.*
import com.oppwa.msa.model.response.CheckoutCreationResponse

import kotlinx.coroutines.ExperimentalCoroutinesApi

private const val STATE_RESOURCE_PATH = "STATE_RESOURCE_PATH"


/**
 * This activity represents the custom UI integration.
 */
class CustomUIActivity : BasePaymentActivity(), ITransactionListener {

    private lateinit var binding: ActivityCustomUiBinding

    private lateinit var checkoutId: String
    private var paymentProvider: OppPaymentProvider? = null
    private var resourcePath: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        resourcePath = savedInstanceState?.getString(STATE_RESOURCE_PATH)

        binding = ActivityCustomUiBinding.inflate(layoutInflater)
        setContentView(binding.root)

        paymentProvider = OppPaymentProvider(this, Connect.ProviderMode.TEST)
        paymentProvider!!.setThreeDSWorkflowListener { this }

        initViews()
    }

    override fun onSaveInstanceState(outState: Bundle, outPersistentState: PersistableBundle) {
        super.onSaveInstanceState(outState, outPersistentState)

        outState.putString(STATE_RESOURCE_PATH, resourcePath)
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)

        setIntent(intent)

        // check of the intent contains the callback scheme
        if (intent.scheme == getString(R.string.custom_ui_callback_scheme)) {
            requestPaymentStatus(resourcePath!!)
        }
    }

    override fun onCheckoutCreated(response: CheckoutCreationResponse?) {
        super.onCheckoutCreated(response)

        response?.ndc.let {
            this.checkoutId = response?.ndc!!
            requestCheckoutInfo(checkoutId)
        }
    }

    override fun paymentConfigRequestSucceeded(checkoutInfo: CheckoutInfo) {
        // get the resource path from checkout info to request the payment status later
        resourcePath = checkoutInfo.resourcePath

        runOnUiThread {
            showConfirmationDialog(checkoutInfo.amount.toString(),
                    checkoutInfo.currencyCode!!
            )
        }
    }

    override fun paymentConfigRequestFailed(paymentError: PaymentError) {
        hideProgressBar()
        showErrorDialog(paymentError)
    }

    @ExperimentalCoroutinesApi
    override fun transactionCompleted(transaction: Transaction) {
        if (transaction.transactionType == TransactionType.SYNC) {
            // check the status of synchronous transaction
            requestPaymentStatus(resourcePath!!)
        } else {
            // open redirect url and wait for the final callback in the onNewIntent()
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(transaction.redirectUrl)))
        }
    }

    override fun transactionFailed(transaction: Transaction, paymentError: PaymentError) {
        hideProgressBar()
        showErrorDialog(paymentError)
    }

    private fun initViews() {
        binding.holderEditText.setText(Constants.Config.CARD_HOLDER_NAME)
        binding.numberEditText.setText(Constants.Config.CARD_NUMBER)
        binding.expiryMonthEditText.setText(Constants.Config.CARD_EXPIRY_MONTH)
        binding.expiryYearEditText.setText(Constants.Config.CARD_EXPIRY_YEAR)
        binding.cvvEditText.setText(Constants.Config.CARD_CVV)
        progressBar = binding.progressBarCustomUi

        binding.buttonPayNow.setOnClickListener {
            if (paymentProvider != null && checkFields()) {
                requestCheckoutId()
            }
        }
    }

    private fun checkFields(): Boolean {
        if (binding.holderEditText.text.isEmpty() ||
            binding.numberEditText.text.isEmpty() ||
            binding.expiryMonthEditText.text.isEmpty() ||
            binding.expiryYearEditText.text.isEmpty() ||
            binding.cvvEditText.text.isEmpty()) {
            showAlertDialog(R.string.error_empty_fields)

            return false
        }

        return true
    }

    private fun requestCheckoutInfo(checkoutId: String) {
        try {
            paymentProvider!!.requestCheckoutInfo(checkoutId, this)
        } catch (e: PaymentException) {
            e.message?.let { showAlertDialog(it) }
        }
    }

    private fun pay(checkoutId: String) {
        try {
            val paymentParams = createPaymentParams(checkoutId)
            paymentParams.shopperResultUrl =
                getString(R.string.custom_ui_callback_scheme) + "://callback"
            val transaction = Transaction(paymentParams)

            paymentProvider!!.submitTransaction(transaction, this)
        } catch (e: PaymentException) {
            showErrorDialog(e.error)
        }
    }

    private fun createPaymentParams(checkoutId: String): CardPaymentParams {
        val cardHolder = binding.holderEditText.text.toString()
        val cardNumber = binding.numberEditText.text.toString()
        val cardExpiryMonth = binding.expiryMonthEditText.text.toString()
        val cardExpiryYear = binding.expiryYearEditText.text.toString()
        val cardCVV = binding.cvvEditText.text.toString()

        return CardPaymentParams(
                checkoutId,
                Constants.Config.CARD_BRAND,
                cardNumber,
                cardHolder,
                cardExpiryMonth,
                "20$cardExpiryYear",
                cardCVV
        )
    }

    private fun showConfirmationDialog(amount: String, currency: String) {
        AlertDialog.Builder(this)
                .setMessage(String.format(
                    getString(R.string.message_payment_confirmation), amount, currency))
                .setPositiveButton(R.string.button_ok) { _, _ -> pay(checkoutId) }
                .setNegativeButton(R.string.button_cancel) { _, _ -> hideProgressBar()}
                .setCancelable(false)
                .show()
    }

    private fun showErrorDialog(message: String) {
        runOnUiThread { showAlertDialog(message) }
    }

    private fun showErrorDialog(paymentError: PaymentError) {
        runOnUiThread { showErrorDialog(paymentError.errorMessage) }
    }
}